combineStep <- function(vari,TLUT,tstep,projinfo) {

  grid <- array(NA, dim=c(length(projinfo$lat.syn),length(projinfo$lon.syn)))
   
  for (i in 1:(length(TLUT)/6)) {
   if ((length(TLUT)/6)==1) {
    if (TLUT[1]<10) H <- paste("0",TLUT[1],sep="") else H <- TLUT[1]
    if (TLUT[2]<10) V <- paste("0",TLUT[2],sep="") else V <- TLUT[2]
   } else {
    if (TLUT[i,1]<10) H <- paste("0",TLUT[i,1],sep="") else H <- TLUT[i,1]
    if (TLUT[i,2]<10) V <- paste("0",TLUT[i,2],sep="") else V <- TLUT[i,2]
   }
   
   # take only tiles which are available
   # TLUT may be large than what is needed for target domain
   if (!file.exists(paste(mpath,vari,".smooth.h",H,"v",V,".",projinfo$year,".ier1km.nc",sep="")))
     next
     
   # load proper time step of tile
   library(RNetCDF)
   nc <- open.nc(paste(mpath,vari,".smooth.h",H,"v",V,".",projinfo$year,".ier1km.nc",sep=""))
   tile <- var.get.nc(nc, "indices", start=c(NA,NA,tstep), count=c(NA,NA,1))    
   tile[round(tile)==(-1E5)] <- NA
   close.nc(nc)
   print(paste("Now processing: ",vari,".smooth.h",H,"v",V,".",projinfo$year,".ier1km.nc",sep=""))

   # first row of tile is NA, replace with second row
   tile[1,] <- tile[2,]

   # east most column may be NA, replace this
   if (is.na(mean(tile[,dim(tile)[2]],na.rm=T))&!is.na(mean(tile[,dim(tile)[2]-1],na.rm=T))) tile[,dim(tile)[2]] <- tile[,dim(tile)[2]-1]

   if ((length(TLUT)/6)==1) {
    LL.lat <- TLUT[3]
    LL.lon <- TLUT[4]
    UR.lat <- TLUT[5]
   } else {
    LL.lat <- TLUT[i,3]
    LL.lon <- TLUT[i,4]
    UR.lat <- TLUT[i,5]    
   }

   # ERRORS HERE MAY BE BECAUSE OF SYNMAP TOO SMALL -> CREATE BIGGER ONE

   # get relative position in y direction
   y.end.grid   <- min(c(dim(grid)[1],which(projinfo$lat.syn>=UR.lat)))
   y.start.grid <- max(c(1,y.end.grid-(1200-(UR.lat-range(projinfo$lat.syn)[2])/spacing)))
   if (((y.end.grid - 1199)>0)&y.end.grid!=dim(grid)[1]) y.start.grid <- y.end.grid - 1199
   if (UR.lat>range(projinfo$lat.syn)[2]) {
    y.end.tile <- 1201-(UR.lat-range(projinfo$lat.syn)[2])/spacing 
   } else {
    y.end.tile <- 1200
   }
   if (LL.lat<range(projinfo$lat.syn)[1]) {
    y.start.tile <- 1200-length(y.start.grid:y.end.grid)+1
   } else {
    y.start.tile <- 1
   } 

   # get relative startings in x direction
   # case 1) tile overlaps beginning of grid
   # case 2) tile doesn't overlap beginning of grid    
   x.start.grid <- round((LL.lon-min(projinfo$lon.syn))/spacing)
   if (x.start.grid < 0) {
    x.start.tile <- x.start.grid*(-1)-1
    x.start.grid <- 1
   } else x.start.tile <- 1

   # get relative endings in x direction
   # case 1) tile overlaps end of grid
   # case 2) tile doesn't overlap end of grid      
   x.end.grid <- x.start.grid+(length((x.start.tile+1):dim(tile)[2]))
   if (x.end.grid > dim(grid)[2]) {
    x.end.grid <- dim(grid)[2]
    #x.end.tile <- length(x.start.grid:dim(grid)[2])
    x.end.tile <- x.start.tile+length(x.start.grid:dim(grid)[2])-1
   } else x.end.tile <- dim(tile)[2]

   ####################################################################
   ## Dim errors here may be result of too small synmap. Check this! ##
   ####################################################################

   # get part of the MODIS tile which is also part of the SYNMAP area (sub tile)
   # sub.tile <- tile[y.start.tile:dim(tile)[1],x.start.tile:x.end.tile]
   sub.tile <- tile[y.start.tile:y.end.tile,x.start.tile:x.end.tile]
   # get matrix with TRUE where output grid has no values
   gridNA <- is.na(grid[y.start.grid:y.end.grid,x.start.grid:x.end.grid])
   # get matrix with TRUE where sub tile has no values
   sub.tileNA <- is.na(sub.tile)
   # replace NAs in output grid with zeros where we have values in sub tile (to make sure addition works since NA+value=NA)
   grid[y.start.grid:y.end.grid,x.start.grid:x.end.grid][gridNA&!sub.tileNA] <- 0
   # add sub tile to output grid
   grid[y.start.grid:y.end.grid,x.start.grid:x.end.grid][!sub.tileNA] <- grid[y.start.grid:y.end.grid,x.start.grid:x.end.grid][!sub.tileNA]+sub.tile[!sub.tileNA]
   # calculate average where values are defined in both the sub tile and the output grid
   grid[y.start.grid:y.end.grid,x.start.grid:x.end.grid][!sub.tileNA&!gridNA] <- grid[y.start.grid:y.end.grid,x.start.grid:x.end.grid][!sub.tileNA&!gridNA]/2    
  } 

  assignr(paste(vari,".",projinfo$year,".",tstep,".combined",sep=""),grid,path=datp)
  
 return(NULL) 
}


# first combine each timestep seperately
# afterwards combine steps
combineTiles <- function(vari="evi",tiles,doyuse,projinfo,fcstmode=FALSE) {

  print(date())
  print("Combining...")

  print(paste("Steps total: ",length(doyuse),sep=""))
  for (tstep in 1:length(doyuse)) {
   if (!fcstmode&file.exists(paste(datp,"/.RData",vari,".",projinfo$year,".",tstep,".combined",sep=""))) {
    print(paste("Step ",tstep," already combined. Skipping.",sep=""))
    next
   }
   combineStep(vari,tiles,tstep,projinfo)
  }

  print("Combining Done.")
  print(date())
  return(NULL)
}


