cutSynmap<- function(dom,projinfo) {

  vpath  <- rdatain
  synmap <- getr(paste("synmap.",synmaptouse,sep=""),vpath) #created in gridEurope.r

  #first reverse
  synmap <- synmap[dim(synmap)[1]:1,]

  #assume upper left corner is id of grid cell
  lat    <- as.numeric(dimnames(synmap)[[1]])
  lat    <- lat-diff(lat)[1] #from upper left to lower left (spacing equivalent)
  lon    <- as.numeric(dimnames(synmap)[[2]])

  # get the right part (where our grid is) of the synmap
  # Project ll and ur coners to LCC.

  if (dom<10) filename <- paste(wrfinput,"/",wrfinname,"0",dom,".nc",sep="")
  if (dom>9 ) filename <- paste(wrfinput,"/",wrfinname,"",dom,".nc",sep="")

  library(RNetCDF)
  nc       <- open.nc(filename)
  lat.wrf  <- try(t(var.get.nc(nc,"XLAT")),silent=TRUE);if(inherits(lat.wrf, "try-error"))lat.wrf  <- t(var.get.nc(nc,"XLAT_M"))
  lon.wrf  <- try(t(var.get.nc(nc,"XLONG")),silent=TRUE);if(inherits(lon.wrf, "try-error"))lon.wrf  <- t(var.get.nc(nc,"XLONG_M"))
  close.nc(nc)
 
  sely <- lat>=min(lat.wrf)&lat<=max(lat.wrf)
  selx <- lon>=min(lon.wrf)&lon<=max(lon.wrf)
  synmapn<-synmap[sely,selx]

  synmapn[synmapn==0]   <- 48 #add water!
  synmap2vprm.key       <- read.table(paste(scriptpath,"/RSources/synmap2vprm.txt",sep=""),header=TRUE)
  synmap2vprm           <- synmap2vprm.key[synmapn,"VPRM"]
  synmap2vprm           <- matrix(synmap2vprm,ncol=dim(synmapn)[2])
  dimnames(synmap2vprm) <- dimnames(synmapn)
  lat                   <- as.numeric(dimnames(synmap2vprm)[[1]])
  lat                   <- lat-diff(lat)[1] #from upper left to lower left (spacing equivalent)
  lon                   <- as.numeric(dimnames(synmap2vprm)[[2]])

  # save for later use
  assignr(paste("vegetationmapCutted.",projinfo$year,".",dom,sep=""),synmap2vprm,datp)

  return(list(synmap2vprm,sely,selx))
}

getDomainMinMax<-function(vari,year,dom,doyuse,synmap2vprm){
  # get minimum indices map
  # to do: take flexible snow times into account
  FINAL   <- getr(paste(vari,".indicesCutted.",year,".1.",dom,sep=""),datp)

  min <- FINAL

  for (tstep in 2:length(doyuse)) {
   FINAL  <- getr(paste(vari,".indicesCutted.",year,".",tstep,".",dom,sep=""),datp)
   min    <- getMinMat(min,FINAL)
  }

  rm(FINAL);gc()

  return(min)
}

classIndicesInTimeStep<-function(vari,year,dom,tstep,minima,water,synmap2vprm) {

  FINAL <- getr(paste(vari,".indicesCutted.",year,".",tstep,".",dom,sep=""),datp)
  
  # replace the NA in winter with minimum indices map
  isna        <- is.na(FINAL)&!water
  FINAL[isna] <- minima[isna]

  # separate vegetation classes
  nveg           <- 8
  indicesClassed <- array(dim=c(dim(synmap2vprm)[1],dim(synmap2vprm)[2],nveg))
  vegs           <- which(1:8%in%synmap2vprm)
  for(v in vegs) {
    indicesClassed[,,v]<-FINAL
    indicesClassed[,,v][synmap2vprm!=v]=NA
  }
  assignr(paste(vari,".indicesClassed.",year,".",tstep,".",dom,sep=""),indicesClassed,datp)

}

cutToDomainInTimeStep<-function(sely,selx,vari,synmap2vprm,dom,tstep,year,doyuse,fcstmode=FALSE) {

  # check if work was already done
  if (!fcstmode&file.exists(paste(datp,"/.RData",vari,".indicesCutted.",year,".",tstep,".",dom,sep=""))) {    
    print(paste(datp,"/.RData",vari,".indicesCutted.",year,".",tstep,".",dom," exists.",sep=""))
    return(NULL)
  }

  # cut part out of combined tiles with same size as vegmap
  grid <- getr(paste(vari,".",year,".",tstep,".combined",sep=""),datp)
  grid <- grid[sely,selx]

  # gap filling with counting of NAs replaced
  water <- synmap2vprm==8
  water[is.na(water)] <- FALSE
  # count NAs over land
  nasbefore <- length(grid[is.na(grid)&!water])
  nasafter  <- nasbefore
  # replace at most 4 times
  i <- 0
  while (nasafter!=0&&i<4) {
   grid    <- na.replace(grid,diagTF=TRUE)
   # count NAs for land
   nasafter <- length(grid[is.na(grid)&!water])
   i <- i+1
  }
  print("Summary of NA replacement (over land):")
  print(date())
  print(paste("Timestep: ",tstep,"; Nest: ",dom,"; Variable: ",vari,sep=""))
  print("NAs before: ");print(nasbefore)
  print("NAs after: ");print(nasafter)
  
  # done. now save results:
  assignr(paste(vari,".indicesCutted.",year,".",tstep,".",dom,sep=""),grid,datp)  
}

cutToDomain<-function(vari="evi",dom=1,doyuse,projinfo,fcstmode=FALSE) {

  skip <- TRUE
  for (tstep in 1:length(doyuse))
   if (!file.exists(paste(datp,"/.RData",vari,".indicesClassed.",projinfo$year,".",tstep,".",dom,sep=""))) skip<-FALSE

  if(fcstmode)skip<-FALSE

  if (!skip||!file.exists(paste(datp,"/.RDatavegetationmapCutted.",projinfo$year,".",dom,sep=""))) {
   synmap      <- cutSynmap(dom,projinfo)
   synmap2vprm <- synmap[[1]]
   sely        <- synmap[[2]]
   selx        <- synmap[[3]]
   rm(synmap);gc()
  }

  if (skip) {
   print(paste("Classed indices for ",toupper(vari)," already there. Skipping.",sep=""))
   return(NULL)
  }

  print(date())
  print(paste("Steps total: ",length(doyuse),sep=""))
  for (tstep in 1:length(doyuse))
   cutToDomainInTimeStep(sely,selx,vari,synmap2vprm,dom,tstep,projinfo$year,doyuse,fcstmode=fcstmode)
   
  # get min/max and class indices; has to happen after cutting all time steps,
  # since min/max is calculated over whole year
  min <- getDomainMinMax(vari,projinfo$year,dom,doyuse,synmap2vprm)

  # get watermask
  water <- synmap2vprm==8
  water[is.na(water)] <- FALSE

  # separate indices in different vegetation classed classes
  # result is written to separate file
  for (tstep in 1:length(doyuse))
   classIndicesInTimeStep(vari,projinfo$year,dom,tstep,min,water,synmap2vprm)

  return(NULL)
}
