#get NRT (near real time) MODIS data
# list of all tiles available according to:
# http://landweb.nascom.nasa.gov/cgi-bin/browse/single.cgi?prod=MOD09A1&sat_ver=Terra_C5&date=2006297&imgProd=MOD09A1&proj=HAM
#
# order: horizontal, vertical
available_tiles <- rbind(
 cbind(13:22,1),

 cbind(10:25,2),

 cbind(7:15,3),
 cbind(17:28,3),

 cbind(8:14,4),
 cbind(17:28,4),
 
 cbind(7:13,5),
 cbind(15:31,5),
 
 cbind(2:4,6),
 cbind(7:9,6),
 cbind(16:32,6),
 
 cbind(1:4,7),
 cbind(6:12,7),
 cbind(15:35,7),
 
 cbind(0:2,8),
 cbind(8:13,8),
 cbind(16:23,8),
 cbind(25:35,8),
 
 cbind(0:5,9),
 cbind(8:14,9),
 cbind(16,9),
 cbind(18:23,9),
 cbind(25,9),
 cbind(27:35,9),
 
 cbind(1:6,10),
 cbind(10:14,10),
 cbind(17,10),
 cbind(19:23,10),
 cbind(26:34,10),
 
 cbind(3:7,11),
 cbind(11:16,11),
 cbind(19:23,11),
 cbind(26:32,11),
 
 cbind(11:13,12),
 cbind(16:17,12),
 cbind(19:20,12),
 cbind(23:30,12),
 
 cbind(7:10,13),
 cbind(12:13,13),
 cbind(19:22,13),
 cbind(25:28,13),
 
 cbind(14:17,14),
 cbind(18,14),
 cbind(20:22,14),
 cbind(24:25,14)
)


checkModisNRT<-function(TLUT,year,doyNRT) {

  ModUsr <- Sys.getenv("ModUsr", unset = NA)  # get MODIS username and password
  ModPw <- Sys.getenv("ModPw", unset = NA)
  #MOD09A1N <- paste("ftp://",ModUsr,":",ModPw,"@nrt4.modaps.eosdis.nasa.gov/allData/6/MOD09A1N/",year,"/",sep="")
  MOD09A1N <- paste("ftp://",ModUsr,":",ModPw,"@nrt3.modaps.eosdis.nasa.gov/allData/6/MOD09A1N/",year,"/",sep="")
 
  intervals <- doyNRT-year*1000


  #check if the dates are available on ftp, otherwise get dates that are close
  # get the list of directories
  require(RCurl)
  require(rgdal)
  items <- try(strsplit(getURL(MOD09A1N,.opts=curlOptions(ftp.response.timeout=2000,connecttimeout=2000), dirlistonly = TRUE), "\n")[[1]])
  #check if there are empty directories
  nitems<-items
  for(item in nitems){
    flsday<-try(strsplit(getURL(paste(MOD09A1N,item,"/",sep=""),.opts=curlOptions(ftp.response.timeout=2000,connecttimeout=2000), dirlistonly = TRUE), "\n")[[1]])
    if(length(flsday)==0)items<-items[!items==item]
  }
  dates<-paste(year,items,sep="")
  julds<-as.numeric(items) #available juldays 
  notthere<-!(intervals%in%julds)
  #Still a problem when directory is there but files have not been updated!

  #get dates that are close, change doyNRT
  for(interval in intervals[notthere]){
    print(paste("day",interval,"not available, checking closest date"))
    intervaln<-julds[abs(julds-interval)==min(abs(julds-interval))]
    print(paste("replacing day",interval,"with day",intervaln))
    doyNRT[intervals==interval]<-intervaln+year*1000
    intervals[intervals==interval]<-intervaln
  }
  intervals<-intervals[!duplicated(intervals)] #duplicated values arise when gap between regular and NRT Modis files is larger than 8 days

  # generate list of needed files
  neededFiles <- array(dim=c(length(TLUT)/6,length(intervals)))
  for (i in 1:(length(TLUT)/6)) {

   if ((length(TLUT)/6)==1) {
    if (TLUT[1]<10) H <- paste("0",TLUT[1],sep="") else H <- TLUT[1]
    if (TLUT[2]<10) V <- paste("0",TLUT[2],sep="") else V <- TLUT[2]
   } else {
    if (TLUT[i,1]<10) H <- paste("0",TLUT[i,1],sep="") else H <- TLUT[i,1]
    if (TLUT[i,2]<10) V <- paste("0",TLUT[i,2],sep="") else V <- TLUT[i,2]
   }

   for (j in 1:length(intervals)) {
    day <- intervals[j]
    if (day<10)       DAY <- paste("00",day,sep="")
    else if (day<100) DAY <- paste("0",day,sep="")
    else              DAY <- day    
    neededFiles[i,j]      <- paste("MOD09A1N.A",year,DAY,".h",H,"v",V,".",modis_ver,sep="")
   }
  }


  # check if files are in modisrep
  if (length(modisrep)!=0) {
   for (i in 1:dim(neededFiles)[1]) {
    for (j in 1:dim(neededFiles)[2]) {
     if (is.na(neededFiles[i,j])) next
     if (length(list.files(path=paste(modisrep,year,sep="/"),pattern=neededFiles[i,j]))>0)
       neededFiles[i,j]<-NA
    }
   }
  }

  # download missing files to via http to mpath
  # note this assumes detailed formatting of usgs website does not change!
  notavail<-NULL

  # For download of MODIS tiles : set the ftdata flag in VPRMpreproc.r
  if (length(neededFiles[!is.na(neededFiles)])!=0&as.logical(ftdata)) {
    nfs<-neededFiles[!is.na(neededFiles)]
    print("Files needed:");print(nfs)
    juldsn<-as.numeric(substring(nfs,15,17)) #needed juldays
    for(juld in unique(juldsn)){
      #check if date is available as directory
      if(!juld%in%julds){print(paste("getMODIS:",nfs[juldsn==juld],"not available"));notavail<-c(notavail,nfs[juldsn==juld])} #date not available
      daten<-dates[julds==juld]
      dayfolder<-items[julds==juld]
      for(nf in nfs[juldsn==juld]){
        getlist <- try(strsplit(getURL(paste(MOD09A1N, dayfolder, "/", sep=""), .opts=c(ftplistonly=TRUE,ftp.response.timeout=2000,connecttimeout=2000,timeout=2000,ssl.verifypeer = FALSE)), "\n")[[1]])
        n.try<-1
        while(inherits(getlist, "try-error")&n.try<80){
          n.try<-n.try+1
          print(paste("trying again",n.try))
          getlist <- try(strsplit(getURL(paste(MOD09A1N, dayfolder, "/", sep=""), .opts=curlOptions(ftplistonly=TRUE,ftp.response.timeout=2000,connecttimeout=2000,timeout=2000,ssl.verifypeer = FALSE)), "\n")[[1]])
        }
        getlist<-getlist[!grepl(".met",getlist)]
        fnm <- getlist[grep(getlist, pattern=paste(nf,".*.hdf",sep=""))[1]]
        if(is.na(fnm)){print(paste("getMODIS:",nf,"not available"));notavail<-c(notavail,nf);next} #date not available
        print(paste("downloading",fnm))
        trycheck<-try(download.file(paste(MOD09A1N, dayfolder, "/", fnm,sep=""), destfile=paste(modisrep,year,fnm, sep="/"), mode='wb', quiet=FALSE, cacheOK=FALSE))
        n.try<-1
        while( (file.info(paste(modisrep,year,fnm, sep="/"))$size<10000|inherits(trycheck, "try-error"))&n.try<80){#something went wrong ... try again
          n.try<-n.try+1
          print(paste("trying again download",n.try))
          trycheck<-try(download.file(paste(MOD09A1N, dayfolder, "/", fnm,sep=""), destfile=paste(modisrep,year,fnm, sep="/"), mode='wb', quiet=FALSE, cacheOK=FALSE))
        }
      }
    }
  }

  # check again if files are in modisrep
  if (length(modisrep)!=0) {
   for (i in 1:dim(neededFiles)[1]) {
    for (j in 1:dim(neededFiles)[2]) {
     if (is.na(neededFiles[i,j])) next
     if (length(list.files(path=paste(modisrep,year,sep="/"),pattern=neededFiles[i,j]))>0)
       neededFiles[i,j]<-NA
    }
   }
  }
  # check if everything is there otherwise stop processing
  if (any(!(neededFiles[!is.na(neededFiles)]%in%notavail))) {
   print("Missing files: ")
   print(neededFiles[!is.na(neededFiles)])
   if (!as.logical(ftdata)) {
    print("or download of MODIS tiles : set ftdata -> TRUE in VPRMpreproc.r")
    }
   stop("Unable to retrieve all MODIS data needed. Aborting.");
   
  }  
  print("all MODIS NRT data downloaded") 
  return(doyNRT)
}




getModisNRT<-function(doyNRT) {

  # load projection info
  projinfo <- getr("projinfo",datp)
  projinfo$year<-year #set this to year specified in config.r

  # get coordinates from WRF
  filename <- paste(wrfinput,"/",wrfinname,"01.nc",sep="")

  library(RNetCDF)
  nc       <- open.nc(filename)
  lat.wrf  <- try(t(var.get.nc(nc,"XLAT")),silent=TRUE);if(inherits(lat.wrf, "try-error"))lat.wrf  <- t(var.get.nc(nc,"XLAT_M"))
  lon.wrf  <- try(t(var.get.nc(nc,"XLONG")),silent=TRUE);if(inherits(lon.wrf, "try-error"))lon.wrf  <- t(var.get.nc(nc,"XLONG_M"))
  close.nc(nc)
   
  ll    <- c(1,2) 
  ur    <- c(1,2)
  ll[1] <- min(lon.wrf)-1
  ll[2] <- min(lat.wrf)-1  
  ur[1] <- max(lon.wrf)+1
  ur[2] <- max(lat.wrf)+1  

  lons <- seq(from=ll[1],to=ur[1])
  lats <- seq(from=ll[2],to=ur[2])
  tiles <- array()
  for (x in 1:length(lons)) {
   for (y in 1:length(lats)) {
    tile <- system(paste(ldope,"bin/tile_id -proj=SIN -lon=",lons[x]," -lat=",lats[y],sep=""),intern=TRUE)
    tile <- as.numeric(strsplit(tile,split=" ")[[1]][5:6]) # tile[1] = h; tile[2] = v
    if (x==1) tiles <- tile
    else tiles <- rbind(tiles,tile)
   }
  }
  tiles <- unique(tiles)
  tiles <- tiles[order(tiles[,1]),]

  # only select available tiles
  if ((length(tiles)/2)==1) {
   available <- FALSE
   for (j in 1:dim(available_tiles)[1]) {
    if (sum(tiles==available_tiles[j,])==2) available <- TRUE
   }
   if (!available) return(NULL)
  } else {
   newtiles <- array()
   for (i in 1:dim(tiles)[1]) {
    for (j in 1:dim(available_tiles)[1]) {
     if (sum(tiles[i,]==available_tiles[j,])==2) newtiles <- rbind(newtiles,tiles[i,])
    }
   }
   tiles <- newtiles[2:dim(newtiles)[1],]
  }
  
  if (file.exists(paste(scriptpath,"/tiles.txt",sep=""))) {
   file.remove(paste(scriptpath,"/tiles.txt",sep=""))
  }
  
  # create tile lookup table TLUT
  if ((length(tiles)/2)==1) {
   TLUT <- c(tiles[1],tiles[2],0,0,0,0)
   if (tiles[1]<10) H <- paste("0",tiles[1],sep="") else H <- as.character(tiles[1])
   if (tiles[2]<10) V <- paste("0",tiles[2],sep="") else V <- as.character(tiles[2])
   system(paste("echo '",H,",",V,"' >> ",scriptpath,"/tiles.txt",sep=""))
  } else {
   for (line in 1:dim(tiles)[1]) {
     i <- tiles[line,1]
     j <- tiles[line,2]
     print(paste("h",i," v",j,sep=""))
     if (i<10) H <- paste("0",i,sep="") else H <- as.character(i)
     if (j<10) V <- paste("0",j,sep="") else V <- as.character(j)
     system(paste("echo '",H,",",V,"' >> ",scriptpath,"/tiles.txt",sep=""))
     # create tile table    
     if (line!=1) TLUT <- rbind(TLUT,c(i,j,0,0,0,0))
     else TLUT <- as.array(c(i,j,0,0,0,0))
   }

   # sort TLUT
   TLUT <- TLUT[order(TLUT[,2]),]
   TLUT <- TLUT[order(TLUT[,1]),]
  }
  
  # use this code in case you specified tiles to use in text file
#  tiles <- read.table(paste(scriptpath,"/tiles.txt",sep=""))
#  TLUT <- array(dim=c(6))
#  for (i in 1:length(tiles[[1]])) {
#   v    <- as.numeric(strsplit(strsplit(as.character(tiles[[1]][i]),"\\.")[[1]][3],"v")[[1]][2])
#   h    <- as.numeric(strsplit(strsplit(strsplit(as.character(tiles[[1]][i]),"\\.")[[1]][3],"v")[[1]][1],"h")[[1]][2])
#   TLUT <- rbind(TLUT,as.array(c(h,v,0,0,0,0))) 
#  }
#  TLUT <- TLUT[2:dim(TLUT)[1],]
#  TLUT <- TLUT[order(TLUT[,2]),]
#  TLUT <- TLUT[order(TLUT[,1]),]

  # make sure all MODIS data is available
  checkModisNRT(TLUT,projinfo$year,doyNRT)
  
  return(TLUT)
}

