parseNamelist<-function() {
  
  # get SYNMAP grid
  synmap <- getr(paste("synmap.",synmaptouse,sep=""),rdatain) #created in gridEurope.r

  #assume upper left corner is id of grid cell
  lat    <- rev(as.numeric(dimnames(synmap)[[1]]))
  lat    <- lat-diff(lat)[1] #from upper left to lower left
  lon    <- as.numeric(dimnames(synmap)[[2]])

  # read wrfinput files get domain settings
  # wrf uses lower left grid cell corner to reference coordinates
  # synmap uses upper left grid cell corner to reference coordinates
  if (file.exists(wrfinput)) {
     print(paste("DEBUG: wrfinput files found in:",wrfinput))
   library(RNetCDF)   
  
   x0 <- y0 <- x1 <- y1 <- dxy <- dyy <- nx <- ny  <- array(0,dim=nests)
   for (dom in 1:nests) {
    if (dom<10) filename <- paste(wrfinput,"/",wrfinname,"0",dom,".nc",sep="")
    if (dom>9 ) filename <- paste(wrfinput,"/",wrfinname,dom,".nc",sep="")

    nc      <- open.nc(filename)
    latx    <- var.get.nc(nc,"XLAT_V")
    longx   <- var.get.nc(nc,"XLONG_U")

    nx[dom] <- att.get.nc(nc,"NC_GLOBAL","WEST-EAST_GRID_DIMENSION")-1
    ny[dom] <- att.get.nc(nc,"NC_GLOBAL","SOUTH-NORTH_GRID_DIMENSION")-1
    
    moadstandlat_1 <- att.get.nc(nc,"NC_GLOBAL","TRUELAT1")
    moadstandlat_2 <- att.get.nc(nc,"NC_GLOBAL","TRUELAT2")
    moadstandlon   <- att.get.nc(nc,"NC_GLOBAL","STAND_LON")    
    moadknownlat   <- att.get.nc(nc,"NC_GLOBAL","MOAD_CEN_LAT")
    moadknownlon   <- att.get.nc(nc,"NC_GLOBAL","CEN_LON")
    
    dxy[dom]     <- att.get.nc(nc,"NC_GLOBAL","DX")/1000
    dyy[dom]     <- att.get.nc(nc,"NC_GLOBAL","DY")/1000

    close.nc(nc)
    
    # 12.08.08: problem using moadknownlon for +lon_0 setting, need to use moadstandlon
    # projout<-paste("+proj=lcc +lat_1=",moadstandlat_1," +lat_2=",moadstandlat_2," +lat_0=",moadknownlat," +lon_0=",moadknownlon,sep="")

    #PROJ.4 Organization
    #
    #  +proj=lcc   +lat_1=Latitude of first standard parallel
    #              +lat_2=Latitude of second standard parallel
    #              +lat_0=Latitude of false origin 
    #              +lon_0=Longitude of false origin
    #              +x_0=False Origin Easting
    #              +y_0=False Origin Northing
    #
    # http://www.remotesensing.org/geotiff/proj_list/lambert_conic_conformal_2sp.html

    projout<-paste("+proj=lcc +lat_1=",moadstandlat_1," +lat_2=",moadstandlat_2," +lat_0=",moadknownlat," +lon_0=",moadstandlon,sep="")
     
    x0[dom] <- as.numeric(project.nice(x.in=min(longx), y.in=min(latx),proj=projout)[[1]][1])
    y0[dom] <- as.numeric(project.nice(x.in=min(longx), y.in=min(latx),proj=projout)[[2]][1])
     
    x1[dom] <- as.numeric(project.nice(x.in=max(longx), y.in=max(latx),proj=projout)[[1]][1])
    y1[dom] <- as.numeric(project.nice(x.in=max(longx), y.in=max(latx),proj=projout)[[2]][1])
   
 
   }
  }
  else
  {
  print(paste("ERROR : no wrfinput files found in:",wrfinput))
  }
 
  assignr("projinfo",list(
  x0=x0,
  y0=y0,
  x1=x1,
  y1=y1,
  nx=nx,
  ny=ny,
  dxy=dxy,
  dyy=dyy,
  projout=projout,
  year=year,
  ldoy=ldoy,
  lat.syn=lat,
  lon.syn=lon),path=datp)
}

