# script to check VPRM preprocessing results
#
# usage: 
# ./check_output.sh

library(RNetCDF)

scriptpath <- getwd() # path of preprocessor
sourcepath <- paste(scriptpath,"/RSources/",sep="")
# read configuration
source(paste(scriptpath,"/config.r",sep=""))
datp <- rdataout
source(paste(scriptpath,"/RSources/getr.r",sep=""))

# load grid configuration
projinfo <- getr("projinfo",datp)


resdir <- paste(outdir,"/","VPRM_input_",domainname,"_",projinfo$year,sep="")


tstep         <- 20
dom           <- 1
vari          <- "evi"
veg           <- 6
tests_total   <- 0
tests_failed  <- 0
tests_success <- 0
nveg <- 8 


doyuse   <- seq(as.numeric(paste(projinfo$year,"041",sep="")),as.numeric(paste(projinfo$year,"305",sep="")),by=8)


print(paste("Domain title:",domainname))
print(paste("Year:",year))
print(paste("MODIS Ver.:",modis_ver))


# after aggregation
for (vari in indices) {
 for (dom in 1:length(projinfo$dxy)) {
 # indices <- getr(paste(vari,".indicesAggregated.",projinfo$year,".",dom,sep=""),datp)
 # map <- getr(paste("mapAggregated.",projinfo$year,".",dom,sep=""),datp)
  if (dom<10) domname <- paste("d0",dom,sep="") else domname <- paste("d",dom,sep="")
  nc <- open.nc(paste(resdir,"/VPRM_input_",toupper(vari),"_",domname,"_",projinfo$year,".nc",sep=""))
  indices <- var.get.nc(nc, vari)
  close.nc(nc)
  nc <- open.nc(paste(resdir,"/VPRM_input_VEG_FRA_",domname,"_",projinfo$year,".nc",sep=""))
  map <- var.get.nc(nc, "vegetation_fraction_map")
  close.nc(nc)
  for (tstep in 1:length(doyuse)) {
   for (v in 1:nveg) {
    matevi <- indices[,,tstep,v]
    matmap <- map[,,v]
    matevi[is.na(matevi)]<-0 
    matevi[matevi>0]<-1
    matmap[matmap>0]<-1
    if (sum(matevi-matmap)>0) {
     print(c("vari","tstep","v","sum"))
     print(c(vari,tstep,v,sum(matevi-matmap)))
    }
   }
  }
 }
}

#image(t(matevi))
#x11();image(t(matmap))
#x11();image(t(matevi-matmap))
#
#matevi[which((matevi-matmap)!=0,arr.ind=T)]
#matmap[which((matevi-matmap)!=0,arr.ind=T)]
#(matevi-matmap)[which((matevi-matmap)!=0,arr.ind=T)]

#
#matevi <- indices[,,tstep,v]
#matmap <- map[,,v]
#matevi[is.na(matevi)]<-0 
#matevi[matevi>0]<-1
#matmap[matmap>0]<-1
#image(matevi)
#x11();image(matmap)
#sum(matevi-matmap)
#x11();image(matevi-matmap)
#

## before aggregation
#for (tstep in 1:34) {
# for (v in 1:8) {
#  synmap2vprm    <- getr(paste("vegetationmapCutted.",projinfo$year,".",dom,sep=""),datp)
#  # after classing
#  indicesClassed <- getr(paste(vari,".indicesClassed.",projinfo$year,".",tstep,".",dom,sep=""),datp)  
#  matmap <- synmap2vprm==v*1
#  matevi <- indicesClassed[,,v]/indicesClassed[,,v]
#  matevi[is.na(matevi)]<-0 
#  print(sum(matevi-matmap))
# }
#}
#
#image(matevi)
#x11();image(matmap)
#
## before classing WRF
#indices <- getr(paste(vari,".indicesCutted.",projinfo$year,".",tstep,".",dom,sep=""),datp)
## before classing STILT
##indices <- getr(paste(vari,".indicesCutted.",projinfo$year,".",tstep,sep=""),datp)
#image(t(indices))
#
## before combining
## load proper time step of tile
#library(RNetCDF)
#nc <- open.nc(paste(mpath,vari,".smooth.h",H,"v",V,".",projinfo$year,".ier1km.nc",sep=""))
#tile <- var.get.nc(nc, "indices", start=c(NA,NA,tstep), count=c(NA,NA,1))    
#tile[round(tile)==(-1E5)] <- NA
#close.nc(nc)
#



# check geo referencing SYNMAP/INDICES
print("Compare spatial referencing SYNMAP vs. indices...")
for (dom in 1:nests) {
 if (dom<10) domname <- paste("d0",dom,sep="") else domname <- paste("d",dom,sep="")
 nc <- open.nc(paste(resdir,"/VPRM_input_",toupper(vari),"_MAX_",domname,"_",projinfo$year,".nc",sep=""))
 output <- var.get.nc(nc, paste(vari,"_max",sep=""))
 close.nc(nc)
 nc <- open.nc(paste(resdir,"/VPRM_input_VEG_FRA_",domname,"_",projinfo$year,".nc",sep=""))
 input <- var.get.nc(nc, "vegetation_fraction_map")
 close.nc(nc)

 tests_total <- tests_total + 1
 if (cor.test(as.array(!output[,,6]),as.array(!input[,,6]))$estimate<0.9) {
  tests_failed <- tests_failed + 1
  print("TEST FAILED!")
  print(paste("   DOM:",dom))
  print(cor.test(as.array(!output[,,6]),as.array(!input[,,6])))
 } else {
  tests_success <- tests_success + 1
  print("Test successful.") 
 }
}

#total_in <- input[,,1]+input[,,2]+input[,,3]+input[,,4]+input[,,5]+input[,,6]+input[,,7]+input[,,8]
#image(input[,,1])
#cor.test(as.array(!output[,,8]),as.array(!input[,,8]))$estimate
#0.9597633
# include error
#tin <- input[,,8]
#tin[,2:49] <- tin[,1:48] # shift one pixel to the south
#input[,,8] <- tin
#cor.test(as.array(!output[,,8]),as.array(!input[,,8]))$estimate 
#
#
#tin <- output[,,6]
#tin[,2:49] <- tin[,1:48] # shift one pixel to the south
#output[,,6] <- tin
#cor.test(as.array(!output[,,6]),as.array(!input[,,6]))$estimate 


print("done.")



### check MIN/MAX values #######################################################################################
print("Sanity check of MINIMUM indices...")
input_min <- 9999
for (dom in 1:nests) {
 for (vari in indices) {
  for (tstep in 1:length(doyuse)) {
   input     <- getr(paste(vari,".indicesClassed.",projinfo$year,".",tstep,".",dom,sep=""),datp)
   input_min <- min(input_min,min(input[input>0],na.rm=T))
  }
  if (dom<10) domname <- paste("d0",dom,sep="") else domname <- paste("d",dom,sep="")
  nc <- open.nc(paste(resdir,"/VPRM_input_",toupper(vari),"_MIN_",domname,"_",projinfo$year,".nc",sep=""))
  output <- var.get.nc(nc, paste(vari,"_min",sep=""))
  close.nc(nc)
  tests_total <- tests_total + 1
  print("Test if index minimum lower than input minimum...")
  if (min(output[output>0])<input_min) {
   tests_failed <- tests_failed + 1
   print("TEST FAILED!")
   print(paste("   DOM: ",dom,"  VARI: ",vari,sep=""))
   print(paste("   min(output[output>0]) -> ",min(output[output>0]),sep=""))
   print(paste("   input_min -> ",input_min,sep=""))
  } else {
   tests_success <- tests_success + 1
   print("Test successful.")
  }
  tests_total <- tests_total + 1
  print("Test if index minimum lower than zero...")
  if (min(output)<0) {
   tests_failed <- tests_failed + 1
   print("TEST FAILED!")
   print(paste("   DOM: ",dom,"  VARI: ",vari,sep=""))
   print(paste("   min(output) -> ",min(output),sep=""))
  } else {
   tests_success <- tests_success + 1
   print("Test successful.")
  }
 }
}
print("done.")


print("Sanity check of MAXIMUM indices...")
input_max <- -9999
for (dom in 1:nests) {
 for (vari in indices) {
  for (tstep in 1:length(doyuse)) {
   input     <- getr(paste(vari,".indicesClassed.",projinfo$year,".",tstep,".",dom,sep=""),datp)
   input_max <- max(input_max,max(input,na.rm=T))
  }
  if (dom<10) domname <- paste("d0",dom,sep="") else domname <- paste("d",dom,sep="")
  nc <- open.nc(paste(resdir,"/VPRM_input_",toupper(vari),"_MAX_",domname,"_",projinfo$year,".nc",sep=""))
  output <- var.get.nc(nc, paste(vari,"_max",sep=""))
  close.nc(nc)
  tests_total <- tests_total + 1
  print("Test if index maximum greater than input maximum...")
  if (max(output)>input_max) {
   tests_failed <- tests_failed + 1
   print("TEST FAILED!")
   print(paste("   DOM: ",dom,"  VARI: ",vari,sep=""))
   print(paste("   max(output) -> ",max(output),sep=""))
   print(paste("   input_max -> ",input_max,sep=""))
  } else {
   tests_success <- tests_success + 1
   print("Test successful.")
  }
  tests_total <- tests_total + 1
  print("Test if index maximum greater than one...")
  if (max(output)>1) {
   tests_failed <- tests_failed + 1
   print("TEST FAILED!")
   print(paste("   DOM: ",dom,"  VARI: ",vari,sep=""))
   print(paste("   max(output) -> ",max(output),sep=""))
  } else {
   tests_success <- tests_success + 1
   print("Test successful.")
  }
 }
}
print("done.")


### check index output in comparison with MODIS #############################################
print("Comparing indices against lowess filtered MODIS...")
for (dom in 1:nests) {
 for (vari in indices) {
  for (tstep in 1:length(doyuse)) {
   input <- getr(paste(vari,".indicesClassed.",projinfo$year,".",tstep,".",dom,sep=""),datp)
   for (veg  in 1:8) {
    
    if (dom<10) domname <- paste("d0",dom,sep="") else domname <- paste("d",dom,sep="")
    nc <- open.nc(paste(resdir,"/VPRM_input_",toupper(vari),"_",domname,"_",projinfo$year,".nc",sep=""))
    output <- var.get.nc(nc, vari)
    close.nc(nc)
    
    # skip if all NA in input
    if (is.na(mean(input[,,veg],na.rm=T))) next
    
    tests_total <- tests_total + 1
    
    if (0.1<abs(mean(input[,,veg],na.rm=T)-mean(output[,,tstep,veg][output[,,tstep,veg]>0]))) {
     tests_failed <- tests_failed + 1
     print("TEST FAILED!")
     print(paste("   DOM: ",dom,"  VARI: ",vari,"  TSTEP: ",tstep,"  VEG: ",veg,sep=""))
     print(paste("   mean(input[,,veg],na.rm=T) -> ",mean(input[,,veg],na.rm=T),sep=""))
     print(paste("   mean(output[,,tstep,veg][output[,,tstep,veg]>0]) -> ",mean(output[,,tstep,veg][output[,,tstep,veg]>0]),sep=""))
     print(paste("   abs(mean(input[,,veg],na.rm=T)-mean(output[,,tstep,veg][output[,,tstep,veg]>0])) -> ",abs(mean(input[,,veg],na.rm=T)-mean(output[,,tstep,veg][output[,,tstep,veg]>0])),sep=""))     
    } else {
     tests_success <- tests_success + 1
     print("Test successful.")
    }
    
   }
  }
 }
}
print("done.")


print(paste("Total tests: ",tests_total,sep=""))
print(paste("Successful tests: ",tests_success,sep=""))
print(paste("FAILED TESTS: ",tests_failed,sep=""))


if (tests_failed==0) quit(status=0) else quit(status=1)
