# script to find regrid mapping for WRF grid on the cluster

scriptpath <- getwd() # path of preprocessor
sourcepath <- paste(scriptpath,"/RSources/",sep="")

source(paste(scriptpath,"/RSources/monthdayyear.r",sep=""))

# read configuration
source(paste(scriptpath,"/config.r",sep=""))

ftdata   <- FALSE          # shall we connect to ftp and check for data?
projtype <- "LCC"          # type of projection LCC xor GEO
datp <- rdataout
mdy      <- month.day.year
attachn  <- function(what, pos = 2){if(!(paste("file:",what,sep="")%in%search()))attach(what,pos)}
go       <- graphics.off

# set synmap resolution
spacing <- 1/120

# set number of used vegetation classes
nveg <- 8 

# create working directories
if (!file.exists(mpath)) dir.create(mpath,recursive=TRUE)
if (!file.exists(rdatain)) dir.create(rdatain,recursive=TRUE)
if (!file.exists(rdataout)) dir.create(rdataout,recursive=TRUE)

# check for synmap
if (!file.exists(paste(rdatain,"/.RDatasynmap.",synmaptouse,sep=""))) {
  #stop(paste("Synmap ",rdatain,"/.RDatasynmap.",synmaptouse," not found. Use ",sourcepath,"/gridEurope.r to obtain synmap!",sep=""));
  print("SYNMAP not there. Prepare one...")
  source(paste(scriptpath,"/RSources/gridEurope.r",sep=""))
  if (!file.exists(paste(rdatain,"/.RDatasynmap.",synmaptouse,sep=""))) {
   stop("Could not prepare SYNMAP!")
  }
}

setwd(rdataout)

#library("hdf5")
library("h5r")

source(paste(scriptpath,"/RSources/assignr.r",sep=""))
source(paste(scriptpath,"/RSources/project.nice.r",sep=""))
source(paste(scriptpath,"/RSources/getminmax.r",sep=""))
source(paste(scriptpath,"/RSources/distance.r",sep=""))
source(paste(scriptpath,"/RSources/getr.r",sep=""))
source(paste(scriptpath,"/RSources/loess.r",sep=""))
source(paste(scriptpath,"/RSources/imagell.r",sep=""))
source(paste(scriptpath,"/RSources/na_replace.r",sep=""))
source(paste(scriptpath,"/RSources/combineTiles.r",sep=""))
source(paste(scriptpath,"/RSources/hdfread.r",sep=""))
source(paste(scriptpath,"/RSources/parseNamelist.r",sep=""))
source(paste(scriptpath,"/RSources/getModis.r",sep=""))
source(paste(scriptpath,"/RSources/extractAndReduce.r",sep=""))
source(paste(scriptpath,"/RSources/doResampling.r",sep=""))
source(paste(scriptpath,"/RSources/readIndicesIntoR.r",sep=""))
source(paste(scriptpath,"/RSources/cutToDomain.r",sep=""))
source(paste(scriptpath,"/RSources/doAggregation.r",sep=""))
source(paste(scriptpath,"/RSources/writeOutput.r",sep=""))

# load grid configuration
projinfo <- getr("projinfo",datp)

# set 8 day intervals to process
# to do: calculate from start and end date set in namelist!
#doyuse <- doyall  <- seq(as.numeric(paste(projinfo$year,"009",sep="")),as.numeric(paste(projinfo$year,"361",sep="")),by=8)
doyuse <- doyall  <- seq(as.numeric(paste(projinfo$year,"001",sep="")),as.numeric(paste(projinfo$year,projinfo$ldoy,sep="")),by=8)
#doyuse   <- seq(as.numeric(paste(projinfo$year,"041",sep="")),as.numeric(paste(projinfo$year,"305",sep="")),by=8)

# cd to working directory
setwd(mpath)



# get from command line
args <- commandArgs()
print(args)
dom         <- as.numeric(args[5])
start_index <- as.numeric(args[6])
count       <- as.numeric(args[7])-1






# get coordinates from WRF
if (dom<10) filename <- paste(wrfinput,"/",wrfinname,"0",dom,".nc",sep="")
if (dom>9 ) filename <- paste(wrfinput,"/",wrfinname,"",dom,".nc",sep="")

# load wrf grid
library(RNetCDF)
nc       <- open.nc(filename)
lat.wrf  <- t(var.get.nc(nc,"XLAT_V"))
lon.wrf  <- t(var.get.nc(nc,"XLONG_U"))  
close.nc(nc)

# aggregate map and indices
synmap <- getr(paste("vegetationmapCutted.",projinfo$year,".",dom,sep=""),datp)
lat    <- as.numeric(dimnames(synmap)[[1]])
lat    <- lat-diff(lat)[1] #from upper left to lower left (spacing equivalent)
lon    <- as.numeric(dimnames(synmap)[[2]])
lon.in <- rep(1,length(lat))%o%lon
lat.in <- lat%o%rep(1,length(lon))

# free memory
rm(synmap);gc()   


mapping <- NULL # ini, inj, outi, outj, sum(sel)


for (i in start_index:(start_index+count)) {
 print(date())

 # get smaller grid to speed up inner loop
 lat.range <- range(lat.wrf[i:(i+1),])
 ind.range <- which(lat.in>=lat.range[1]&lat.in<=lat.range[2],arr.ind=T)
 if (length(ind.range)==0) break
 row.range <- range(ind.range[,1])
 row.range[2] <- max(2,row.range[2])
 if (row.range[1]==row.range[2]) {
   if ( row.range[1]>2 ) row.range[1]<-row.range[1]-1 else row.range[2]<-row.range[1]+1
 }
 lat.small <- lat.in[row.range[1]:row.range[2],]
 lon.small <- lon.in[row.range[1]:row.range[2],]
 offset    <- row.range[1]-1  

 for (j in 1:(dim(lon.wrf)[2]-1)) {
  #print(paste(j,date()))
  sel <- lat.small>=lat.wrf[i,j] & lat.small<lat.wrf[i+1,j] & lon.small>=lon.wrf[i,j] & lon.small<lon.wrf[i,j+1]
  if (sum(sel)>0) {
   selection <- which(sel==TRUE,arr.ind=T)
   selection <- cbind(selection[,1]+offset,selection[,2])
   mapping <- rbind(mapping,cbind(selection,i,j,sum(sel)))
  }
 }
 print(paste(i/((start_index+count))*100,"%"))    
}

assignr(paste("lsf_mapping.LCC.",dom,"_idx",start_index,sep=""), mapping, datp)


